from enum import Enum


class ItemType(Enum):
    DOTNET = "dotnet"
    ELIXIR = "elixir"
    GOLANG = "golang"
    HASKELL = "haskell"
    JAVA = "java"
    JAVASCRIPT = "javascript"
    JULIA = "julia"
    MAKE = "make"
    MELTANO = "meltano"
    PHP = "php"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
